import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Deposit } from '../models/deposit';
import { DepositPagination } from '../models/deposit-pagination';
import { DepositService } from '../services/deposit.service';

@Injectable({
  providedIn: 'root',
})
export class DepositResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _depositService: DepositService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Deposit | undefined> {
    return this._depositService.getDepositById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class DepositsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _depositService: DepositService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: DepositPagination;
    deposits: Deposit[];
  }> {
    return this._depositService.getDeposits();
  }
}
