import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { WebCalculate } from '../models/web-calculate';
import { WebCalculatePagination } from '../models/web-calculate-pagination';
import { WebCalculateService } from '../services/web-calculate.service';

@Injectable({
  providedIn: 'root',
})
export class WebCalculateResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _webCalculateService: WebCalculateService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<WebCalculate | undefined> {
    return this._webCalculateService
      .getWebCalculateById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class WebCalculatesResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _webCalculateService: WebCalculateService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: WebCalculatePagination;
    webCalculates: WebCalculate[];
  }> {
    return this._webCalculateService.getWebCalculates();
  }
}
