import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Powerball } from '../models/powerball';
import { PowerballPagination } from '../models/powerball-pagination';
import { PowerballService } from '../services/powerball.service';

@Injectable({
  providedIn: 'root',
})
export class PowerballResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _powerballService: PowerballService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Powerball | undefined> {
    return this._powerballService
      .getPowerballById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class PowerballsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _powerballService: PowerballService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: PowerballPagination;
    powerballs: Powerball[];
  }> {
    return this._powerballService.getPowerballs();
  }
}
