import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { CasinoMoney } from '../models/casino-money';
import { CasinoMoneyPagination } from '../models/casino-money-pagination';
import { CasinoMoneyService } from '../services/casino-money.service';

@Injectable({
  providedIn: 'root',
})
export class CasinoMoneyResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _casinoMoneyService: CasinoMoneyService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<CasinoMoney | undefined> {
    return this._casinoMoneyService
      .getCasinoMoneyById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CasinoMoneysResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _casinoMoneyService: CasinoMoneyService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CasinoMoneyPagination;
    casinoMoneys: CasinoMoney[];
  }> {
    return this._casinoMoneyService.getCasinoMoneys();
  }
}
