import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { CouponLog } from '../models/coupon-log';
import { CouponLogPagination } from '../models/coupon-log-pagination';
import { CouponLogService } from '../services/coupon-log.service';

@Injectable({
  providedIn: 'root',
})
export class CouponLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _couponLogService: CouponLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<CouponLog | undefined> {
    return this._couponLogService
      .getCouponLogById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CouponLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _couponLogService: CouponLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CouponLogPagination;
    couponLogs: CouponLog[];
  }> {
    return this._couponLogService.getCouponLogs();
  }
}
