import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Coupon } from '../models/coupon';
import { CouponPagination } from '../models/coupon-pagination';
import { CouponService } from '../services/coupon.service';

@Injectable({
  providedIn: 'root',
})
export class CouponResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _couponService: CouponService, private _router: Router) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Coupon | undefined> {
    return this._couponService.getCouponById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CouponsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _couponService: CouponService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CouponPagination;
    coupons: Coupon[];
  }> {
    return this._couponService.getCoupons();
  }
}
