import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Unconnected } from '../models/unconnected';
import { UnconnectedPagination } from '../models/unconnected-pagination';
import { UnconnectedService } from '../services/unconnected.service';

@Injectable({
  providedIn: 'root',
})
export class UnconnectedResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _unconnectedService: UnconnectedService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Unconnected | undefined> {
    return this._unconnectedService
      .getUnconnectedById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class UnconnectedsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _unconnectedService: UnconnectedService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: UnconnectedPagination;
    unconnecteds: Unconnected[];
  }> {
    return this._unconnectedService.getUnconnecteds();
  }
}
