import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { AdminSession } from '../models/admin-session';
import { AdminSessionPagination } from '../models/admin-session-pagination';
import { AdminSessionService } from '../services/admin-session.service';

@Injectable({
  providedIn: 'root',
})
export class AdminSessionResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _adminSessionService: AdminSessionService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<AdminSession | undefined> {
    return this._adminSessionService
      .getAdminSessionById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class AdminSessionsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _adminSessionService: AdminSessionService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: AdminSessionPagination;
    adminSessions: AdminSession[];
  }> {
    return this._adminSessionService.getAdminSessions();
  }
}
