import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { CompLog } from '../models/comp-log';
import { CompLogPagination } from '../models/comp-log-pagination';
import { CompLogService } from '../services/comp-log.service';

@Injectable({
  providedIn: 'root',
})
export class CompLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _compLogService: CompLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<CompLog | undefined> {
    return this._compLogService.getCompLogById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CompLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _compLogService: CompLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CompLogPagination;
    compLogs: CompLog[];
  }> {
    return this._compLogService.getCompLogs();
  }
}
