import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { DuplicatedSession } from '../models/duplicated-session';
import { DuplicatedSessionPagination } from '../models/duplicated-session-pagination';
import { DuplicatedSessionService } from '../services/duplicated-session.service';

@Injectable({
  providedIn: 'root',
})
export class DuplicatedSessionResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _duplicatedSessionService: DuplicatedSessionService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<DuplicatedSession | undefined> {
    return this._duplicatedSessionService
      .getDuplicatedSessionById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class DuplicatedSessionsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _duplicatedSessionService: DuplicatedSessionService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: DuplicatedSessionPagination;
    duplicatedSessions: DuplicatedSession[];
  }> {
    return this._duplicatedSessionService.getDuplicatedSessions();
  }
}
