import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { User } from '../../../member/user/models/user';
import { ExcelLog } from '../models/excel-log';
import { ExcelLogPagination } from '../models/excel-log-pagination';
import { ExcelLogService } from '../services/excel-log.service';
import { Router } from '@angular/router';

@Component({
  selector: 'excel-log-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .excel-log-grid {
        grid-template-columns: 50px 300px auto 200px;

        @screen sm {
          grid-template-columns: 50px 300px auto 200px;
        }

        @screen md {
          grid-template-columns: 50px 300px auto 200px;
        }

        @screen lg {
          grid-template-columns: 50px 300px auto 200px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  excelLogs$!: Observable<ExcelLog[] | undefined>;

  isLoading = false;
  searchInputControl = new FormControl();
  selectedExcelLog?: ExcelLog;
  pagination?: ExcelLogPagination;
  __isSearchOpened = false;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _excelLogService: ExcelLogService,
    private router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._excelLogService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: ExcelLogPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this.excelLogs$ = this._excelLogService.excelLogs$;
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {}

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch(): void {
    this.__isSearchOpened = !this.__isSearchOpened;
  }

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }
}
