import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { ExcelLog } from '../models/excel-log';
import { ExcelLogPagination } from '../models/excel-log-pagination';
import { ExcelLogService } from '../services/excel-log.service';

@Injectable({
  providedIn: 'root',
})
export class ExcelLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _excelLogService: ExcelLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<ExcelLog | undefined> {
    return this._excelLogService.getExcelLogById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class ExcelLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _excelLogService: ExcelLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: ExcelLogPagination;
    excelLogs: ExcelLog[];
  }> {
    return this._excelLogService.getExcelLogs();
  }
}
