import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { ModificationLog } from '../models/modification-log';
import { ModificationLogPagination } from '../models/modification-log-pagination';
import { ModificationLogService } from '../services/modification-log.service';

@Injectable({
  providedIn: 'root',
})
export class ModificationLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _modificationLogService: ModificationLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<ModificationLog | undefined> {
    return this._modificationLogService
      .getModificationLogById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class ModificationLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _modificationLogService: ModificationLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: ModificationLogPagination;
    modificationLogs: ModificationLog[];
  }> {
    return this._modificationLogService.getModificationLogs();
  }
}
