import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { MoneyLog } from '../models/money-log';
import { MoneyLogPagination } from '../models/money-log-pagination';
import { MoneyLogService } from '../services/money-log.service';

@Injectable({
  providedIn: 'root',
})
export class MoneyLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _moneyLogService: MoneyLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<MoneyLog | undefined> {
    return this._moneyLogService.getMoneyLogById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class MoneyLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _moneyLogService: MoneyLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: MoneyLogPagination;
    moneyLogs: MoneyLog[];
  }> {
    return this._moneyLogService.getMoneyLogs();
  }
}
