import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { PaymentLog } from '../models/payment-log';
import { PaymentLogPagination } from '../models/payment-log-pagination';
import { PaymentLogService } from '../services/payment-log.service';

@Injectable({
  providedIn: 'root',
})
export class PaymentLogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _paymentLogService: PaymentLogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<PaymentLog | undefined> {
    return this._paymentLogService
      .getPaymentLogById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class PaymentLogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _paymentLogService: PaymentLogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: PaymentLogPagination;
    paymentLogs: PaymentLog[];
  }> {
    return this._paymentLogService.getPaymentLogs();
  }
}
