import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { UserSession } from '../models/user-session';
import { UserSessionPagination } from '../models/user-session-pagination';
import { UserSessionService } from '../services/user-session.service';

@Injectable({
  providedIn: 'root',
})
export class UserSessionResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _userSessionService: UserSessionService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<UserSession | undefined> {
    return this._userSessionService
      .getUserSessionById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class UserSessionsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _userSessionService: UserSessionService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: UserSessionPagination;
    userSessions: UserSession[];
  }> {
    return this._userSessionService.getUserSessions();
  }
}
