import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { BasicSetting } from '../models/basic-setting';
import { BasicService } from '../services/basic.service';
import { CurrentUser } from 'app/modules/admin/member/current-user/models/current-user';
import { User } from 'app/modules/admin/member/user/models/user';
import { MatTableDataSource } from '@angular/material/table';

@Component({
  selector: 'settings-basic',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .settings-basic-grid {
        grid-template-columns: 60px auto 40px;

        @screen sm {
          grid-template-columns: 60px auto 60px 72px;
        }

        @screen md {
          grid-template-columns: 60px 60px auto 112px 72px;
        }

        @screen lg {
          grid-template-columns: 60px 60px auto 112px 96px 96px 72px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  currentUsers$!: Observable<CurrentUser[] | undefined>;
  users$!: Observable<User[] | undefined>;

  siteSettingTableColumns: string[] = ['info', 'description', 'updateBtn'];

  siteEarnSettingTableColumns: string[] = [
    'level',
    'firstRate',
    'firstIsUse',
    'firstMaxMoney',
    'everyRate',
    'everyIsUse',
    'everyMaxMoney',
    'updateBtn',
  ];
  siteGameSettingTableColumns: string[] = [
    'title',
    'isUse',
    'memo',
    'description',
    'updateBtn',
  ];

  basicInfoSettingDataSource: MatTableDataSource<any> =
    new MatTableDataSource();
  basicEarnSettingDataSource: MatTableDataSource<any> =
    new MatTableDataSource();
  basicGameSettingDataSource: MatTableDataSource<any> =
    new MatTableDataSource();

  basicSetting$!: Observable<BasicSetting | undefined>;

  isLoading = false;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _basicService: BasicService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the products
    this._basicService.basicSetting$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((basicSetting: any | undefined) => {
        this.basicInfoSettingDataSource = basicSetting.infos;
        this.basicEarnSettingDataSource = basicSetting.earnSettings;
        this.basicGameSettingDataSource = basicSetting.gameSetting;
        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this.basicSetting$ = this._basicService.basicSetting$;
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {}

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }
}
